@@ -1,7 +1,7 @@
 #' Individual Unit Root Tests without multiple testing control
 #' @description This function performs bootstrap unit root tests on each time series individually.
 #' @param data A \eqn{T}-dimensional vector or a (\eqn{T} x \eqn{N})-matrix of \eqn{N} time series with \eqn{T} observations to be tested for unit roots. Data may also be in a time series format (e.g. \code{ts}, \code{zoo} or \code{xts}), or a data frame, as long as each column represents a single time series.
-#' @param level Desired significance level of the unit root test. Default is 0.05.
+#' @param data_name Optional name for the data, to be used in the output. The default uses the name of the 'data' argument.
 #' @param bootstrap String for bootstrap method to be used. Options are
 #' \describe{
 #' \item{\code{"MBB"}}{Moving block bootstrap (Paparoditis and Politis, 2003; Palm, Smeekes and Urbain, 2011);}
@@ -14,10 +14,9 @@
 #' @param B Number of bootstrap replications. Default is 1999.
 #' @param block_length Desired 'block length' in the bootstrap. For the MBB, BWB and DWB bootstrap, this is a genuine block length. For the AWB bootstrap, the block length is transformed into an autoregressive parameter via the formula \eqn{0.01^(1/block_length)} as in Smeekes and Urbain (2014a); this can be overwritten by setting \code{ar_AWB} directly. Default sets the block length as a function of the time series length T, via the rule \eqn{block_length = 1.75 T^(1/3)} of Palm, Smeekes and Urbain (2011).
 #' @param ar_AWB Autoregressive parameter used in the AWB bootstrap method (\code{bootstrap = "AWB"}). Can be used to set the parameter directly rather than via the default link to the block length.
+#' @param union_quantile The quantile of the bootstrap distribution used for scaling the individual statistics in the union. Ideally this should equal the desired significance level of the test. Default is 0.05. This parameter is overwritten when a significance level is provided in the argument \code{level}.
+#' @param level The desired significance level of the test (optional). This is only used for multivariate series to be able to provide a boolean vector with rejections of the null hypothesis or not for easy post-processing. Default is \code{NULL}, in which case no such vector is given.
 #' @param union Logical indicator whether or not to use bootstrap union tests (\code{TRUE}) or not (\code{FALSE}), see Smeekes and Taylor (2012). Default is \code{TRUE}.
-#' @param min_lag Minimum lag length in the augmented Dickey-Fuller regression. Default is 0.
-#' @param max_lag Maximum lag length in the augmented Dickey-Fuller regression. Default uses the sample size-based rule \eqn{12(T/100)^{1/4}}.
-#' @param criterion String for information criterion used to select the lag length in the augmented Dickey-Fuller regression. Options are: \code{"AIC"}, \code{"BIC"}, \code{"MAIC"}, \code{"MBIC"}. Default is \code{"MAIC"} (Ng and Perron, 2001).
 #' @param deterministics String indicating the deterministic specification. Only relevant if \code{union = FALSE}. Options are
 #'
 #' \verb{"none":} no deterministics;
@@ -28,11 +27,13 @@
 #'
 #' If \code{union = FALSE}, the default is adding an intercept (a warning is given).
 #' @param detrend String indicating the type of detrending to be performed. Only relevant if \code{union = FALSE}. Options are: \code{"OLS"} or \code{"QD"} (typically also called GLS, see Elliott, Rothenberg and Stock, 1996). The default is \code{"OLS"}.
+#' @param min_lag Minimum lag length in the augmented Dickey-Fuller regression. Default is 0.
+#' @param max_lag Maximum lag length in the augmented Dickey-Fuller regression. Default uses the sample size-based rule \eqn{12(T/100)^{1/4}}.
+#' @param criterion String for information criterion used to select the lag length in the augmented Dickey-Fuller regression. Options are: \code{"AIC"}, \code{"BIC"}, \code{"MAIC"}, \code{"MBIC"}. Default is \code{"MAIC"} (Ng and Perron, 2001).
 #' @param criterion_scale Logical indicator whether or not to use the rescaled information criteria of Cavaliere et al. (2015) (\code{TRUE}) or not (\code{FALSE}). Default is \code{TRUE}.
 #' @param show_progress Logical indicator whether a bootstrap progress update should be printed to the console. Default is FALSE.
-#' @param do_parallel Logical indicator whether bootstrap loop should be executed in parallel. Parallel computing is only available if OpenMP can be used, if not this option is ignored. Default is FALSE.
+#' @param do_parallel Logical indicator whether bootstrap loop should be executed in parallel. Default is TRUE.
 #' @param cores The number of cores to be used in the parallel loops. Default is to use all but one.
-#' @param data_name Optional name for the data, to be used in the output. The default uses the name of the 'data' argument.
 #' @details The options encompass many test proposed in the literature. \code{detrend = "OLS"} gives the standard augmented Dickey-Fuller test, while \code{detrend = "QD"} provides the DF-GLS test of Elliott, Rothenberg and Stock (1996). The bootstrap algorithm is always based on a residual bootstrap (under the alternative) to obtain residuals rather than a difference-based bootstrap (under the null), see e.g. Palm, Smeekes and Urbain (2008).
 #'
 #' Lag length selection is done automatically in the ADF regression with the specified information criterion. If one of the modified criteria of Ng and Perron (2001) is used, the correction of Perron and Qu (2008) is applied. For very short time series (fewer than 50 time points) the maximum lag length is adjusted downward to avoid potential multicollinearity issues in the bootstrap. To overwrite data-driven lag length selection with a pre-specified lag length, simply set both the minimum `min_lag` and maximum lag length `max_lag` for the selection algorithm equal to the desired lag length.
@@ -42,10 +43,10 @@
 #' \item{\code{data.name}}{The name of the data on which the method is performed;}
 #' \item{\code{null.value}}{The value of the (gamma) parameter of the lagged dependent variable in the ADF regression under the null hypothesis. Under the null, the series has a unit root. Testing the null of a unit root then boils down to testing the significance of the gamma parameter;}
 #' \item{\code{alternative}}{A character string specifying the direction of the alternative hypothesis relative to the null value. The alternative postulates that the series is stationary;}
-#' \item{\code{estimate}}{The estimated value(s) of the (gamma) parameter of the lagged dependent variable in the ADF regressions. Note that for the union test (\code{union = TRUE}), this estimate is not defined, hence NA is returned;}
+#' \item{\code{estimate}}{The estimated value(s) of the (gamma) parameter of the lagged dependent variable in the ADF regressions. Note that for the union test (\code{union = TRUE}), this estimate is not defined, hence \code{NA} is returned;}
 #' \item{\code{statistic}}{The value(s) of the test statistic of the unit root test(s);}
 #' \item{\code{p.value}}{The p-value(s) of the unit root test(s);}
-#' \item{\code{rejections}}{For \code{"mult_htest"} only. A vector with logical indicators for each time series whether the null hypothesis of a unit root is rejected (\code{TRUE}) or not (\code{FALSE});}
+#' \item{\code{rejections}}{For \code{"mult_htest"} only. A vector with logical indicators for each time series whether the null hypothesis of a unit root is rejected (\code{TRUE}) or not (\code{FALSE}). This is only supplied when an optional significance level is given, otherwise \code{NULL} is returned;}
 #' \item{\code{details}}{For \code{"mult_htest"} only. The details of the performed tests in a matrix containing parameter estimate. test statistic and p-value for each time series.}
 #' \item{\code{series.names}}{For \code{"mult_htest"} only. The names of the series that the tests are performed on.}
 #' @section Warnings:
@@ -74,17 +75,21 @@
 #' @references Smeekes, S. and Urbain, J.-P. (2014a). A multivariate invariance principle for modified wild bootstrap methods with an application to unit root testing. GSBE Research Memorandum No. RM/14/008, Maastricht University
 #' @examples
 #' # boot_ur on GDP_BE and GDP_DE
-#' two_series_boot_ur <- boot_ur(MacroTS[, 1:2], bootstrap = "MBB", B = 399)
+#' two_series_boot_ur <- boot_ur(MacroTS[, 1:2], bootstrap = "MBB", B = 199,
+#'                               do_parallel = FALSE, show_progress = FALSE)
 #' print(two_series_boot_ur)
 #' @export
-boot_ur <- function(data, level = 0.05, bootstrap = "AWB", B = 1999, block_length = NULL,
-                    ar_AWB = NULL, union = TRUE, min_lag = 0, max_lag = NULL,
-                    criterion = "MAIC", deterministics = NULL, detrend = NULL,
-                    criterion_scale = TRUE, show_progress = TRUE, do_parallel = FALSE,
-                    cores = NULL, data_name = NULL){
+boot_ur <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999, block_length = NULL,
+                    ar_AWB = NULL, level = NULL, union = TRUE, union_quantile = 0.05,
+                    deterministics = NULL, detrend = NULL, min_lag = 0, max_lag = NULL,
+                    criterion = "MAIC", criterion_scale = TRUE, show_progress = TRUE,
+                    do_parallel = TRUE, cores = NULL){
 
+  if (!is.null(level)) {
+    union_quantile <- level
+  }
   inputs <- do_tests_and_bootstrap(data = data, boot_sqt_test = FALSE, boot_ur_test = TRUE,
-                                   level = level, bootstrap = bootstrap, B = B,
+                                   level = union_quantile, bootstrap = bootstrap, B = B,
                                    block_length = block_length, ar_AWB = ar_AWB,
                                    union = union, min_lag = min_lag, max_lag = max_lag,
                                    criterion = criterion, deterministics = deterministics,
@@ -128,7 +133,7 @@ boot_ur <- function(data, level = 0.05, bootstrap = "AWB", B = 1999, block_lengt
     iADFout <- cbind(t(inputs$param_i), iADFout) # Parameter estimates, tstats and p-values
     if (NCOL(data) > 1) {
       rownames(iADFout) <- var_names
-      colnames(iADFout) <- c("gamma", "tstat", "p-value")
+      colnames(iADFout) <- c("gamma", "statistic", "p.value")
     }
 
     if (NCOL(data) == 1){ # boot_adf
@@ -138,7 +143,11 @@ boot_ur <- function(data, level = 0.05, bootstrap = "AWB", B = 1999, block_lengt
     }
   }
 
-  rej_H0 <- (iADFout[, 3] < level)
+  if (!is.null(level)) {
+    rej_H0 <- (iADFout[, 3] < level)
+  } else {
+    rej_H0 <- NULL
+  }
 
   if (NCOL(data) > 1) {
     boot_ur_output <- list(method = method_name, data.name = data_name,
@@ -148,12 +157,15 @@ boot_ur <- function(data, level = 0.05, bootstrap = "AWB", B = 1999, block_lengt
                            details = NULL, series.names = var_names)
     class(boot_ur_output) <- c("bootUR", "mult_htest")
   } else {
+    param <- drop(iADFout[1, 1])
+    attr(param, "names") <- "gamma"
     iADFtstat <- iADFout[1, 2]
     attr(iADFtstat, "names") <- "tstat"
+    p_val <- drop(iADFout[1, 3])
+    attr(p_val, "names") <- "p-value"
     boot_ur_output <- list(method = method_name, data.name = var_names,
                            null.value = c("gamma" = 0), alternative = "less",
-                           estimate = iADFout[1, 1], statistic = iADFtstat,
-                           p.value = iADFout[1, 3])
+                           estimate = param, statistic = iADFtstat, p.value = p_val)
     class(boot_ur_output) <- c("bootUR", "htest")
   }
 
@@ -208,14 +220,14 @@ boot_ur <- function(data, level = 0.05, bootstrap = "AWB", B = 1999, block_lengt
 #' @seealso \code{\link{boot_ur}}
 #' @examples
 #' # boot_adf on GDP_BE
-#' GDP_BE_adf <- boot_adf(MacroTS[, 1], B = 399, deterministics = "trend",
-#' detrend = "OLS")
+#' GDP_BE_adf <- boot_adf(MacroTS[, 1], B = 199, deterministics = "trend", detrend = "OLS",
+#'                        do_parallel = FALSE, show_progress = FALSE)
 #' print(GDP_BE_adf)
-boot_adf <- function(data, level = 0.05, bootstrap = "AWB", B = 1999,
-                     block_length = NULL, ar_AWB = NULL, min_lag = 0, max_lag = NULL,
-                     criterion = "MAIC", deterministics = "intercept",
+boot_adf <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999,
+                     block_length = NULL, ar_AWB = NULL, deterministics = "intercept",
+                     min_lag = 0, max_lag = NULL, criterion = "MAIC",
                      detrend = "OLS", criterion_scale = TRUE, show_progress = TRUE,
-                     do_parallel = FALSE, cores = NULL, data_name = NULL){
+                     do_parallel = TRUE, cores = NULL){
 
   if (NCOL(data) > 1) {
     stop("Multiple time series not allowed. Switch to a multivariate method such as boot_ur,
@@ -228,7 +240,7 @@ boot_adf <- function(data, level = 0.05, bootstrap = "AWB", B = 1999,
       data_name <- colnames(data)[1]
     }
   }
-  out <- boot_ur(data = data, level = level, bootstrap = bootstrap, B = B,
+  out <- boot_ur(data = data, bootstrap = bootstrap, B = B,
                  block_length = block_length, ar_AWB = ar_AWB, union = FALSE,
                  min_lag = min_lag, max_lag = max_lag, criterion = criterion,
                  deterministics = deterministics, detrend = detrend,
@@ -287,12 +299,12 @@ boot_adf <- function(data, level = 0.05, bootstrap = "AWB", B = 1999,
 #' @seealso \code{\link{boot_ur}}
 #' @examples
 #' # boot_union on GDP_BE
-#' GDP_BE_df <- boot_union(MacroTS[, 1], B = 399)
+#' GDP_BE_df <- boot_union(MacroTS[, 1], B = 199, do_parallel = FALSE, show_progress = FALSE)
 #' print(GDP_BE_df)
-boot_union <- function(data, level = 0.05, bootstrap = "AWB", B = 1999, block_length = NULL,
+boot_union <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999, block_length = NULL,
                        ar_AWB = NULL, min_lag = 0, max_lag = NULL, criterion = "MAIC",
-                       criterion_scale = TRUE, show_progress = TRUE, do_parallel = FALSE,
-                       cores = NULL, data_name = NULL){
+                       criterion_scale = TRUE, union_quantile = 0.05, show_progress = TRUE,
+                       do_parallel = TRUE, cores = NULL){
 
   if (NCOL(data) > 1) {
     stop("Multiple time series not allowed. Switch to a multivariate method such as boot_ur,
@@ -305,7 +317,7 @@ boot_union <- function(data, level = 0.05, bootstrap = "AWB", B = 1999, block_le
       data_name <- colnames(data)[1]
     }
   }
-  out <- boot_ur(data = data, level = level, bootstrap = bootstrap, B = B,
+  out <- boot_ur(data = data, union_quantile = union_quantile, bootstrap = bootstrap, B = B,
                  block_length = block_length, ar_AWB = ar_AWB, union = TRUE, min_lag = min_lag,
                  max_lag = max_lag, criterion = criterion, criterion_scale = criterion_scale,
                  show_progress = show_progress, do_parallel = do_parallel,
@@ -317,7 +329,7 @@ boot_union <- function(data, level = 0.05, bootstrap = "AWB", B = 1999, block_le
 #' Bootstrap Unit Root Tests with False Discovery Rate control
 #' @description Controls for multiple testing by controlling the false discovery rate (FDR), see Moon and Perron (2012) and Romano, Shaikh and Wolf (2008).
 #' @inheritParams boot_ur
-#' @param level Desired False Discovery Rate level of the unit root tests. Default is 0.05.
+#' @param FDR_level Desired False Discovery Rate level of the unit root tests. Default is 0.05.
 #' @param union Logical indicator whether or not to use bootstrap union tests (\code{TRUE}) or not (\code{FALSE}), see Smeekes and Taylor (2012). Default is \code{TRUE}.
 #' @details The false discovery rate FDR is defined as the expected proportion of false rejections relative to the total number of rejections.
 #'
@@ -363,17 +375,18 @@ boot_union <- function(data, level = 0.05, bootstrap = "AWB", B = 1999, block_le
 #' @seealso \code{\link{boot_ur}}
 #' @examples
 #' # boot_fdr on GDP_BE and GDP_DE
-#' two_series_boot_fdr <- boot_fdr(MacroTS[, 1:2], bootstrap = "MBB", B = 399)
+#' two_series_boot_fdr <- boot_fdr(MacroTS[, 1:2], bootstrap = "MBB", B = 199,
+#'                                 do_parallel = FALSE, show_progress = FALSE)
 #' print(two_series_boot_fdr)
 #' @export
-boot_fdr <- function(data, level = 0.05,  bootstrap = "AWB", B = 1999, block_length = NULL,
-                     ar_AWB = NULL, union = TRUE, min_lag = 0, max_lag = NULL,
-                     criterion = "MAIC", deterministics = NULL, detrend = NULL,
-                     criterion_scale = TRUE, show_progress = TRUE, do_parallel = FALSE,
-                     cores = NULL, data_name = NULL){
+boot_fdr <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999, block_length = NULL,
+                     ar_AWB = NULL, FDR_level = 0.05, union = TRUE, deterministics = NULL,
+                     detrend = NULL, min_lag = 0, max_lag = NULL, criterion = "MAIC",
+                     criterion_scale = TRUE, show_progress = TRUE, do_parallel = TRUE,
+                     cores = NULL){
 
   inputs <- do_tests_and_bootstrap(data = data, boot_sqt_test = FALSE, boot_ur_test = FALSE,
-                                   level = level, bootstrap = bootstrap, B = B,
+                                   level = FDR_level, bootstrap = bootstrap, B = B,
                                    block_length = block_length, ar_AWB = ar_AWB, union = union,
                                    min_lag = min_lag, max_lag = max_lag, criterion = criterion,
                                    deterministics = deterministics, detrend = detrend,
@@ -398,13 +411,13 @@ boot_fdr <- function(data, level = 0.05,  bootstrap = "AWB", B = 1999, block_len
     bFDRout <- FDR_cpp(test_i = inputs$test_stats, t_star = inputs$test_stats_star,
                        level = inputs$level)
     estimates <- rep(NA, NCOL(data))
-    tstats <- inputs$test_stats
+    tstats <- drop(inputs$test_stats)
     method_name <- "Bootstrap Union Tests with False Discovery Rate control"
   } else { # No Union Tests
       bFDRout <- FDR_cpp(test_i = matrix(inputs$tests_i[1, ], nrow = 1),
                          t_star = inputs$t_star[ , 1,], level = inputs$level)
       estimates <- t(inputs$param_i)
-      tstats <- inputs$tests_i[1, ]
+      tstats <- drop(inputs$tests_i[1, ])
       method_name <- "Bootstrap ADF Tests with False Discovery Rate control"
   }
   rej_H0 <- matrix(bFDRout$rej_H0 == 1, nrow = NCOL(data))
@@ -413,10 +426,12 @@ boot_fdr <- function(data, level = 0.05,  bootstrap = "AWB", B = 1999, block_len
   FDR_seq <- bFDRout$FDR_Tests[, -1, drop = FALSE]
   rownames(FDR_seq) <- var_names[bFDRout$FDR_Tests[, 1, drop = FALSE]]
   colnames(FDR_seq) <- c("tstat", "critical value")
+  p_vals <- rep(NA, NCOL(data))
+  names(estimates) <- names(tstats) <- names(p_vals) <- var_names
 
   fdr_output <- list(method = method_name, data.name = data_name,
                      null.value =  c("gamma" = 0), alternative = "less",
-                     estimate = estimates, statistic = tstats, p.value = rep(NA, NCOL(data)),
+                     estimate = estimates, statistic = tstats, p.value = p_vals,
                      rejections = rej_H0, details = FDR_seq, series.names = var_names)
   class(fdr_output) <- c("bootUR", "mult_htest")
 
@@ -426,6 +441,7 @@ boot_fdr <- function(data, level = 0.05,  bootstrap = "AWB", B = 1999, block_len
 #' Bootstrap Sequential Quantile Test
 #' @description Performs the Bootstrap Sequential Quantile Test (BSQT) proposed by Smeekes (2015).
 #' @inheritParams boot_ur
+#' @param SQT_level Desired significance level of the sequential tests performed. Default is 0.05.
 #' @param steps Numeric vector of quantiles or units to be tested. Default is to test each unit sequentially.
 #' @param union Logical indicator whether or not to use bootstrap union tests (\code{TRUE}) or not (\code{FALSE}), see Smeekes and Taylor (2012). Default is \code{TRUE}.
 #' @details The parameter \code{steps} can either be set as an increasing sequence of integers smaller or equal to the number of series \code{N}, or fractions of the total number of series (quantiles). For \code{N} time series, setting \code{steps = 0:N} means each unit should be tested sequentially. In this case the method is equivalent to the StepM method of Romano and Wolf (2005), and therefore controls the familywise error rate. To split the series in \code{K} equally sized groups, use \code{steps = 0:K / K}.
@@ -475,17 +491,18 @@ boot_fdr <- function(data, level = 0.05,  bootstrap = "AWB", B = 1999, block_len
 #' @seealso \code{\link{boot_ur}}
 #' @examples
 #' # boot_sqt on GDP_BE and GDP_DE
-#' two_series_boot_sqt <- boot_sqt(MacroTS[, 1:2], bootstrap = "AWB", B = 399)
+#' two_series_boot_sqt <- boot_sqt(MacroTS[, 1:2], bootstrap = "AWB", B = 199,
+#'                                 do_parallel = FALSE, show_progress = FALSE)
 #' print(two_series_boot_sqt)
 #' @export
-boot_sqt <- function(data, steps = 0:NCOL(data), level = 0.05,  bootstrap = "AWB",
-                     B = 1999, block_length = NULL, ar_AWB = NULL, union = TRUE,
-                     min_lag = 0, max_lag = NULL, criterion = "MAIC", deterministics = NULL,
-                     detrend = NULL, criterion_scale = TRUE, show_progress = TRUE,
-                     do_parallel = FALSE, cores = NULL, data_name = NULL){
+boot_sqt <- function(data, data_name = NULL, steps = 0:NCOL(data), bootstrap = "AWB",
+                     B = 1999, block_length = NULL, ar_AWB = NULL, SQT_level = 0.05, union = TRUE,
+                     deterministics = NULL, detrend = NULL, min_lag = 0, max_lag = NULL,
+                     criterion = "MAIC", criterion_scale = TRUE, show_progress = TRUE,
+                     do_parallel = TRUE, cores = NULL){
 
   inputs <- do_tests_and_bootstrap(data = data, boot_sqt_test = TRUE, boot_ur_test = FALSE,
-                                   level = level, bootstrap = bootstrap, B = B,
+                                   level = SQT_level, bootstrap = bootstrap, B = B,
                                    block_length = block_length, ar_AWB = ar_AWB, union = union,
                                    min_lag = min_lag, max_lag = max_lag, criterion = criterion,
                                    deterministics = deterministics, detrend = detrend,
@@ -510,13 +527,13 @@ boot_sqt <- function(data, steps = 0:NCOL(data), level = 0.05,  bootstrap = "AWB
     BSQTout <- BSQT_cpp(pvec = inputs$p_vec, test_i = inputs$test_stats,
                         t_star = inputs$test_stats_star, level = inputs$level)
     estimates <- rep(NA, NCOL(data))
-    tstats <- inputs$test_stats
+    tstats <- drop(inputs$test_stats)
     method_name <- "Bootstrap Sequential Quantile Union Test"
   } else { # No Union Tests
     BSQTout <- BSQT_cpp(pvec = inputs$p_vec, test_i = matrix(inputs$tests_i[1, ], nrow = 1),
                         t_star = inputs$t_star[ , 1,], level = inputs$level)
     estimates <- t(inputs$param_i)
-    tstats <- inputs$tests_i[1, ]
+    tstats <- drop(inputs$tests_i[1, ])
     method_name <- "Bootstrap Sequential Quantile ADF Test"
   }
   rej_H0 <- matrix(BSQTout$rej_H0 == 1, nrow = NCOL(data))
@@ -525,10 +542,12 @@ boot_sqt <- function(data, steps = 0:NCOL(data), level = 0.05,  bootstrap = "AWB
   BSQT_seq <- BSQTout$BSQT_steps[, -3, drop = FALSE]
   rownames(BSQT_seq) <- paste("Step", 1:nrow(BSQT_seq))
   colnames(BSQT_seq) <- c("H0: # I(0)", "H1: # I(0)", "tstat", "p-value")
+  p_vals <- rep(NA, NCOL(data))
+  names(estimates) <- names(tstats) <- names(p_vals) <- var_names
 
   sqt_output <- list(method = method_name, data.name = data_name,
                      null.value =  c("gamma" = 0), alternative = "less",
-                     estimate = estimates, statistic = tstats, p.value = rep(NA, NCOL(data)),
+                     estimate = estimates, statistic = tstats, p.value = p_vals,
                      rejections = rej_H0, details = BSQT_seq, series.names = var_names)
   class(sqt_output) <- c("bootUR", "mult_htest")
   return(sqt_output)
@@ -575,18 +594,18 @@ boot_sqt <- function(data, steps = 0:NCOL(data), level = 0.05,  bootstrap = "AWB
 #' @seealso \code{\link{boot_ur}}
 #' @examples
 #' # boot_panel on GDP_BE and GDP_DE
-#' two_series_boot_panel <- boot_panel(MacroTS[, 1:2], bootstrap = "AWB", B = 399)
+#' two_series_boot_panel <- boot_panel(MacroTS[, 1:2], bootstrap = "AWB", B = 199,
+#'                                     do_parallel = FALSE, show_progress = FALSE)
 #' print(two_series_boot_panel)
 #' @export
-boot_panel <- function(data, level = 0.05, bootstrap = "AWB", B = 1999,
-                       block_length = NULL, ar_AWB = NULL, union = TRUE,
-                       min_lag = 0, max_lag = NULL, criterion = "MAIC",
-                       deterministics = NULL, detrend = NULL, criterion_scale = TRUE,
-                       show_progress = TRUE, do_parallel = FALSE, cores = NULL,
-                       data_name = NULL){
+boot_panel <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999,
+                       block_length = NULL, ar_AWB = NULL, union = TRUE, union_quantile = 0.05,
+                       deterministics = NULL, detrend = NULL, min_lag = 0, max_lag = NULL,
+                       criterion = "MAIC", criterion_scale = TRUE, show_progress = TRUE,
+                       do_parallel = TRUE, cores = NULL){
 
   inputs <- do_tests_and_bootstrap(data = data, boot_sqt_test = FALSE, boot_ur_test = FALSE,
-                                   level = level, bootstrap = bootstrap, B = B,
+                                   level = union_quantile, bootstrap = bootstrap, B = B,
                                    block_length = block_length, ar_AWB = ar_AWB, union = union,
                                    min_lag = min_lag, max_lag = max_lag, criterion = criterion,
                                    deterministics = deterministics, detrend = detrend,
@@ -612,6 +631,7 @@ boot_panel <- function(data, level = 0.05, bootstrap = "AWB", B = 1999,
   attr(GM_test, "names") <- "tstat"
   gamma_hat <- NA
   attr(gamma_hat, "names") <- "gamma"
+  attr(p_val, "names") <- "p-value"
   panel_output <- list(method = method_name, data.name = data_name,
                        null.value = c("gamma" = 0), alternative = "less",
                        estimate = gamma_hat, statistic = GM_test, p.value = p_val)
